@@ -1,4 +1,4 @@
-// Copyright 2019 Parity Technologies (UK) Ltd.
+// Copyright 2019-2021 Parity Technologies (UK) Ltd.
 //
 // Permission is hereby granted, free of charge, to any
 // person obtaining a copy of this software and associated
@@ -37,7 +37,8 @@ mod server;
 pub mod response;
 
 pub use access_control::{
-	hosts::{AllowHosts, Host},
+	cors::{AccessControlAllowHeaders, AccessControlAllowOrigin},
+	hosts::{AllowHosts, DomainsValidation, Host},
 	AccessControl, AccessControlBuilder,
 };
 pub use jsonrpsee_types as types;
