@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Parity Technologies (UK) Ltd.
+// Copyright 2019-2021 Parity Technologies (UK) Ltd.
 //
 // Permission is hereby granted, free of charge, to any
 // person obtaining a copy of this software and associated
@@ -31,24 +31,18 @@ use std::net::SocketAddr;
 use crate::types::error::{CallError, Error};
 use crate::{server::StopHandle, HttpServerBuilder, RpcModule};
 
-use futures_util::FutureExt;
 use jsonrpsee_test_utils::helpers::*;
-use jsonrpsee_test_utils::types::{Id, StatusCode, TestContext};
+use jsonrpsee_test_utils::mocks::{Id, StatusCode, TestContext};
 use jsonrpsee_test_utils::TimeoutFutureExt;
 use serde_json::Value as JsonValue;
-use tokio::task::JoinHandle;
 
-async fn server() -> SocketAddr {
-	server_with_handles().await.0
-}
-
-async fn server_with_handles() -> (SocketAddr, JoinHandle<Result<(), Error>>, StopHandle) {
+async fn server() -> (SocketAddr, StopHandle) {
 	let server = HttpServerBuilder::default().build("127.0.0.1:0".parse().unwrap()).unwrap();
 	let ctx = TestContext;
 	let mut module = RpcModule::new(ctx);
 	let addr = server.local_addr().unwrap();
 	module.register_method("say_hello", |_, _| Ok("lo")).unwrap();
-	module.register_async_method("say_hello_async", |_, _| async move { Ok("lo") }.boxed()).unwrap();
+	module.register_async_method("say_hello_async", |_, _| async move { Ok("lo") }).unwrap();
 	module
 		.register_method("add", |params, _| {
 			let params: Vec<u64> = params.parse()?;
@@ -66,36 +60,32 @@ async fn server_with_handles() -> (SocketAddr, JoinHandle<Result<(), Error>>, St
 	module.register_method("notif", |_, _| Ok("")).unwrap();
 	module
 		.register_method("should_err", |_, ctx| {
-			let _ = ctx.err().map_err(|e| CallError::Failed(e.into()))?;
+			let _ = ctx.err().map_err(CallError::Failed)?;
 			Ok("err")
 		})
 		.unwrap();
 
 	module
 		.register_method("should_ok", |_, ctx| {
-			let _ = ctx.ok().map_err(|e| CallError::Failed(e.into()))?;
+			let _ = ctx.ok().map_err(CallError::Failed)?;
 			Ok("ok")
 		})
 		.unwrap();
 	module
-		.register_async_method("should_ok_async", |_p, ctx| {
-			async move {
-				let _ = ctx.ok().map_err(|e| CallError::Failed(e.into()))?;
-				Ok("ok")
-			}
-			.boxed()
+		.register_async_method("should_ok_async", |_p, ctx| async move {
+			let _ = ctx.ok().map_err(CallError::Failed)?;
+			Ok("ok")
 		})
 		.unwrap();
 
-	let stop_handle = server.stop_handle();
-	let join_handle = tokio::spawn(async move { server.start(module).with_default_timeout().await.unwrap() });
-	(addr, join_handle, stop_handle)
+	let stop_handle = server.start(module).unwrap();
+	(addr, stop_handle)
 }
 
 #[tokio::test]
 async fn single_method_call_works() {
 	let _ = env_logger::try_init();
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	for i in 0..10 {
@@ -109,7 +99,7 @@ async fn single_method_call_works() {
 #[tokio::test]
 async fn async_method_call_works() {
 	let _ = env_logger::try_init();
-	let addr = server().await;
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	for i in 0..10 {
@@ -123,7 +113,7 @@ async fn async_method_call_works() {
 #[tokio::test]
 async fn invalid_single_method_call() {
 	let _ = env_logger::try_init();
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":1, "params": "bar"}"#;
@@ -134,7 +124,7 @@ async fn invalid_single_method_call() {
 
 #[tokio::test]
 async fn single_method_call_with_params() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"add", "params":[1, 2],"id":1}"#;
@@ -145,7 +135,7 @@ async fn single_method_call_with_params() {
 
 #[tokio::test]
 async fn single_method_call_with_multiple_params_of_different_types() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"multiparam", "params":["Hello", "World", [0,1,2,3]],"id":1}"#;
@@ -156,18 +146,19 @@ async fn single_method_call_with_multiple_params_of_different_types() {
 
 #[tokio::test]
 async fn single_method_call_with_faulty_params_returns_err() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
+	let expected = r#"{"jsonrpc":"2.0","error":{"code":-32602,"message":"invalid type: string \"this should be a number\", expected u64 at line 1 column 26"},"id":1}"#;
 
-	let req = r#"{"jsonrpc":"2.0","method":"add", "params":["Invalid"],"id":1}"#;
+	let req = r#"{"jsonrpc":"2.0","method":"add", "params":["this should be a number"],"id":1}"#;
 	let response = http_request(req.into(), uri).with_default_timeout().await.unwrap().unwrap();
 	assert_eq!(response.status, StatusCode::OK);
-	assert_eq!(response.body, invalid_params(Id::Num(1)));
+	assert_eq!(response.body, expected);
 }
 
 #[tokio::test]
 async fn single_method_call_with_faulty_context() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"should_err", "params":[],"id":1}"#;
@@ -178,7 +169,7 @@ async fn single_method_call_with_faulty_context() {
 
 #[tokio::test]
 async fn single_method_call_with_ok_context() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"should_ok", "params":[],"id":1}"#;
@@ -189,7 +180,7 @@ async fn single_method_call_with_ok_context() {
 
 #[tokio::test]
 async fn async_method_call_with_ok_context() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"should_ok_async", "params":[],"id":1}"#;
@@ -202,7 +193,7 @@ async fn async_method_call_with_ok_context() {
 async fn valid_batched_method_calls() {
 	let _ = env_logger::try_init();
 
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"[
@@ -223,7 +214,7 @@ async fn valid_batched_method_calls() {
 async fn batched_notifications() {
 	let _ = env_logger::try_init();
 
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"[{"jsonrpc": "2.0", "method": "notif", "params": [1,2,4]},{"jsonrpc": "2.0", "method": "notif", "params": [7]}]"#;
@@ -237,7 +228,7 @@ async fn batched_notifications() {
 async fn invalid_batched_method_calls() {
 	let _ = env_logger::try_init();
 
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	// batch with no requests
@@ -270,9 +261,55 @@ async fn invalid_batched_method_calls() {
 	assert_eq!(response.body, parse_error(Id::Null));
 }
 
+#[tokio::test]
+async fn garbage_request_fails() {
+	let (addr, _handle) = server().await;
+	let uri = to_http_uri(addr);
+
+	let req = r#"dsdfs fsdsfds"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#"{ "#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#"         {"jsonrpc":"2.0","method":"add", "params":[1, 2],"id":1}"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#"{}"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#"{sds}"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#"["#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#"[dsds]"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#" [{"jsonrpc":"2.0","method":"add", "params":[1, 2],"id":1}]"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+
+	let req = r#"[]"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, invalid_request(Id::Null));
+
+	let req = r#"[{"jsonrpc":"2.0","method":"add", "params":[1, 2],"id":1}"#;
+	let response = http_request(req.into(), uri.clone()).await.unwrap();
+	assert_eq!(response.body, parse_error(Id::Null));
+}
+
 #[tokio::test]
 async fn should_return_method_not_found() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"bar","id":"foo"}"#;
@@ -283,18 +320,19 @@ async fn should_return_method_not_found() {
 
 #[tokio::test]
 async fn invalid_json_id_missing_value() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"say_hello","id"}"#;
 	let response = http_request(req.into(), uri).with_default_timeout().await.unwrap().unwrap();
-	// If there was an error in detecting the id in the Request object (e.g. Parse error/Invalid Request), it MUST be Null.
+	// If there was an error in detecting the id in the Request object (e.g. Parse error/Invalid Request), it MUST be
+	// Null.
 	assert_eq!(response.body, parse_error(Id::Null));
 }
 
 #[tokio::test]
 async fn invalid_request_object() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"bar","id":1,"is_not_request_object":1}"#;
@@ -305,7 +343,7 @@ async fn invalid_request_object() {
 
 #[tokio::test]
 async fn notif_works() {
-	let addr = server().with_default_timeout().await.unwrap();
+	let (addr, _handle) = server().with_default_timeout().await.unwrap();
 	let uri = to_http_uri(addr);
 
 	let req = r#"{"jsonrpc":"2.0","method":"bar"}"#;
@@ -342,19 +380,6 @@ async fn can_register_modules() {
 #[tokio::test]
 async fn stop_works() {
 	let _ = env_logger::try_init();
-	let (_addr, join_handle, mut stop_handle) = server_with_handles().with_default_timeout().await.unwrap();
-	stop_handle.stop().with_default_timeout().await.unwrap().unwrap();
-	stop_handle.wait_for_stop().with_default_timeout().await.unwrap();
-
-	// After that we should be able to wait for task handle to finish.
-	// First `unwrap` is timeout, second is `JoinHandle`'s one, third is the server future result.
-	join_handle
-		.with_default_timeout()
-		.await
-		.expect("Timeout")
-		.expect("Join error")
-		.expect("Server stopped with an error");
-
-	// After server was stopped, attempt to stop it again should result in an error.
-	assert!(matches!(stop_handle.stop().with_default_timeout().await.unwrap(), Err(Error::AlreadyStopped)));
+	let (_addr, stop_handle) = server().with_default_timeout().await.unwrap();
+	assert!(matches!(stop_handle.stop().unwrap().await, Ok(_)));
 }
